{ TCapture version 1.0 copyright  1996 by Alexander Meeder

  This visual component is derived from the TImage-component. It provides a simple way
  to capture the entire screen, after that you can save it to an bitmap-image or you can
  make a selection of the captured screen with the mouse and save it.

  properties:
  * HideMyself (true/false) capture screen with or without yourself

  methods:
  * Capture -> Capture1.Capture
  * SaveToFile -> Capture1.SaveToFile(const FileName: string) saves complete screen
  * SaveSelectionToFile -> Capture1.SaveSelectionToFile(const FileName: string) save selection
                  to file, after you made a selection

  events:
  * MouseDown -> set selection startingpoint
  * MouseMove -> draw rectangle to show selected region
  * MouseUp -> set selection endingpoint

  If you have suggestions about TCapture or make changes, pleas mail me: ameeder@dds.nl
}

unit capture;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls;

type
  TCapture = class(TImage)
  private
    FHideMyself: boolean;
    FAlign: TAlign;
    FAutoSize: boolean;
    FCenter: boolean;
    FStretch: boolean;

    procedure VerifyRect;
    procedure DrawMarquee(mStart, mStop: TPoint; AMode: TPenMode);
  protected
    DeskTopDC: HDc;
    DeskTopCanvas: TCanvas;
    DeskTopRect: TRect;
    SelectionBitmap: TBitmap;
    SelectionRect: TRect;

    Selecting: boolean;
    ptOrigin, ptMove: TPoint;

    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: integer); override;
    procedure Loaded; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Capture;
    procedure SaveToFile(const FileName: string);
    procedure SaveSelectionToFile(const FileName: string);
  published
    property HideMyself: boolean read FHideMyself write FHideMyself default true;
    property Align: TAlign read FAlign;              // Hide this property, else it won't work
    property AutoSize: boolean read FAutoSize;       // idem
    property Center: boolean read FCenter;           // idem
    property Stretch: boolean read FStretch;         // idem
  end;

procedure Register;

implementation

// If someone make a selection starting right/bottom and ending left/top swap value's
procedure TCapture.VerifyRect;
var
  TempVar: integer;
begin
  if SelectionRect.Left > SelectionRect.Right then
  begin
    TempVar := SelectionRect.Left;
    SelectionRect.Left := SelectionRect.Right;
    SelectionRect.Right := TempVar;
  end;
  if SelectionRect.Top > SelectionRect.Bottom then
  begin
    TempVar := SelectionRect.Top;
    SelectionRect.Top := SelectionRect.Bottom;
    SelectionRect.Bottom := TempVar;
  end;
end;

// Routine to draw selection-rectangle
procedure TCapture.DrawMarquee(mStart, mStop: TPoint; AMode: TPenMode);
begin
  Canvas.Pen.Mode := AMode;
  Canvas.Rectangle(mStart.X, mStart.Y, mStop.X, mStop.Y);
end;

// Routine to save the selection
procedure TCapture.SaveSelectionToFile(const FileName: string);
begin
  VerifyRect;
  SelectionBitmap := TBitmap.Create;
  try
    SelectionBitmap.Width := SelectionRect.Right - SelectionRect.Left;
    SelectionBitmap.Height := SelectionRect.Bottom - SelectionRect.Top;
    SelectionBitmap.Canvas.CopyRect(Rect(0, 0, SelectionBitmap.Width, SelectionBitmap.Height), Canvas, SelectionRect);
    SelectionBitmap.SaveToFile(FileName);
  finally
    SelectionBitmap.Free;
  end;
end;

// Does what it says
procedure TCapture.SaveToFile(const FileName: string);
begin
  Picture.SaveToFile(FileName);
end;

// Routine to capture the entire screen
procedure TCapture.Capture;
begin
  if FHideMyself then
  begin
    ShowWindow(Application.Handle, SW_HIDE);
    Parent.Hide;
  end;
  DeskTopDC := GetWindowDC(GetDeskTopWindow);
  DeskTopCanvas := TCanvas.Create;
  try
    DeskTopCanvas.Handle := DeskTopDC;
    DeskTopRect := Rect(0, 0, Screen.Width, Screen.Height);
    Width := DeskTopRect.Right - DeskTopRect.Left;
    Height := DeskTopRect.Bottom - DeskTopRect.Top;
    Canvas.CopyRect(DeskTopRect, DeskTopCanvas, DeskTopRect);
  finally
    ReleaseDC(GetDeskTopWindow, DeskTopDC);
    DeskTopCanvas.Free;
  end;
  if FHideMyself then
  begin
    ShowWindow(Application.Handle, SW_SHOW);
    Parent.Show;
  end;
end;

// Select startingpoint of selection and of selection-rectangle
procedure TCapture.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: integer);
begin
  inherited MouseDown(Button, Shift, X, Y);
  Selecting := true;
  ptOrigin := Point(X, Y);
  ptMove := Point(X, Y);
  with canvas do
  begin
    Pen.Color := clBlack;
    Pen.Width := 1;
    Pen.Style := psDashDotDot;
    Brush.Style := bsClear;
  end;

  DrawMarquee(ptOrigin, ptMove, pmNotXor);

  SelectionRect.Left := X;
  SelectionRect.Top := Y;
end;

// Draw selection-rectangle while moving the mouse when selecting
procedure TCapture.MouseMove(Shift: TShiftState; X, Y: Integer);
begin
  inherited MouseMove(Shift, X, Y);
  if Selecting then
  begin
    DrawMarquee(ptOrigin, ptMove, pmNotXor);
    DrawMarquee(ptOrigin, Point(X, Y), pmNotXor);
    ptMove := Point(X, Y);
    Canvas.Pen.Mode := pmCopy;
  end;
end;

// Set selection endingpoint and remove selection-rectangle
procedure TCapture.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: integer);
begin
  inherited MouseUp(Button, Shift, X, Y);
  if Selecting then
  begin
    Selecting := false;
    DrawMarquee(ptOrigin, Point(X, Y), pmNotXor);
    ptMove := Point(X, Y);
    SelectionRect.Right := X;
    SelectionRect.Bottom := Y;
  end;
end;

// Width/Height must be the same as screendimensions
procedure TCapture.Loaded;
begin
  inherited Loaded;
  if not (csDesigning in ComponentState) then
  begin
    Width := Screen.Width;
    Height := Screen.Height;
  end;
end;

// Initialize some values
constructor TCapture.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FHideMyself := true;
  Selecting := false;
end;

destructor TCapture.Destroy;
begin
  inherited Destroy;
end;

procedure Register;
begin
  RegisterComponents('Samples', [TCapture]);
end;

end.
